#!python
'''
filename: symdata.py
function: the independent elastic constant for different symmeties
author: Mingqing Liao
e-mail: liaomq1900127@163.com
FGMS group @ Harbin Institute of Technology(HIT)

The coefficients were taken from
    2nd and 3rd:  K. Brugger, J. Appl. Phys., 36(1965), 759-768
    4th: R. Brendel, Acta Crystallogr a., 35(1979), 525-533
    5th: Chung D Y, Li Y., Acta Crystallogr a., 1974,30(1):1-13.
    
Please cite corresponding reference when using this script
'''

import numpy as np

def coef_crystal(CrystalType="c1", Ord=3):
    '''
    The coef data of the relationship between different elastic constants
        (This function should always combined with coef_ind function,
        When combined these two dataset, the relationship between different
        elastic constant is full specified)
        e.g. CrystalType='H1', Ord=2
            The coef in the last element(C66) is A=[0.5, 0.5], 
            and the ind in the coef_ind function is A=[1, 2], hence, we have
            C66 = 0.5*C11 + 0.5*C12
    Parameter
    ---------
        CrystalType: str
            The crystal type, casse insensitive (Both capital or lower case are ok)
                Support for all symmetries (Using Laue's notation)
                    N: triclinic or n
                    M: monoclinic or m
                    O: orthorhombic or o
                    TI: tetragonal1 or t1
                    TII: tetagonal2 or t2
                    R1: rhombohedral1 or r1
                    R2: rhombohedral2 or r2
                    HI: hexagonal1 or h1
                    HII: hexagonal or h2
                    CI: cubic1 or c1
                    CII: cubic2 or c2
        Ord: int
            The order of elastic constants
            2: full supported
            3: full suported
            4: full supported
            5: partial supported (C, T, O, M, N)
    Return
    ------
        CoefCoef: list
            The coef under specific crystal type and order
    '''
    CrystalType = CrystalType.lower()
    if Ord == 2:
        A = [0.5,-0.5]
        if CrystalType == "triclinic" or CrystalType == "n":
            CoefCoef = [1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1]
        elif CrystalType == "monoclinic" or CrystalType == "m":
            CoefCoef = [1,1,1,0,1,0,1,1,0,1,0,1,0,1,0,1,0,1,1,0,1]
        elif CrystalType == "orthorhombic" or CrystalType == "o":
            CoefCoef = [1,1,1,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1]
        elif CrystalType == "tetragonal2" or CrystalType == "t2":
            CoefCoef = [1,1,1,0,0,1,1,1,0,0,-1,1,0,0,0,1,0,0,1,0,1]
        elif CrystalType == "tetragonal1" or CrystalType == "t1":
            CoefCoef = [1,1,1,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1]        
        elif CrystalType == "rhombohedral2" or CrystalType == "r2":
            CoefCoef = [1,1,1,1,1,0,1,1,-1,-1,0,1,0,0,0,1,0,-1,1,1,A]
        elif CrystalType == "rhombohedral1" or CrystalType == "r1":
            CoefCoef = [1,1,1,1,0,0,1,1,-1,0,0,1,0,0,0,1,0,0,1,1,A]
        elif CrystalType == "hexagonal2" or CrystalType == "h2":
            CoefCoef = [1,1,1,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,A]
        elif CrystalType == "hexagonal1" or CrystalType == "h1":
            CoefCoef = [1,1,1,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,A]        
        elif CrystalType == "cubic2" or CrystalType == "c2":
            CoefCoef = [1,1,1,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1]
        elif CrystalType == "cubic1" or CrystalType == "c1":
            CoefCoef = [1,1,1,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1]
        elif CrystalType == "isotropic" or CrystalType == "i":
            CoefCoef = [1,1,1,0,0,0,1,1,0,0,0,1,0,0,0,A,0,0,A,0,A]
    elif Ord == 3:
        A = [1,1,-1];B = [-0.5,-1.5];C = [0.5,1.5];D = [-0.5,-0.25,0.75];E = [-1,-2];F = [-1,-2]
        G = [-0.5,0.5];H = [0.5,-0.5];I = [0.5,-0.25,-0.25];J = [0.5,-0.5];K = [-0.5,0.5]
        L = [0.5,-0.5];M = [0.25,-0.25];N = [0.125,-0.375,0.25]
        if CrystalType == "triclinic" or CrystalType == "n":
            CoefCoef = [1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
                1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1]
        elif CrystalType == "monoclinic" or CrystalType == "m":
            CoefCoef = [1,1,1,0,1,0,1,1,0,1,0,1,0,1,0,1,0,1,1,0,1,1,1,0,1,0,1,0,1,0,
                1,0,1,1,0,1,1,0,1,0,1,0,1,1,0,1,0,1,0,0,1,0,1,0,1,0]
        elif CrystalType == "orthorhombic" or CrystalType == "o":
            CoefCoef = [1,1,1,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,1,0,0,0,1,0,0,0,
                1,0,0,1,0,1,1,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,0]
        elif CrystalType == "tetragonal2" or CrystalType == "t2":
            CoefCoef = [1,1,1,0,0,1,1,1,0,0,0,1,0,0,1,1,1,0,1,0,1,1,1,0,0,-1,1,0,0,-1,
                1,-1,0,1,0,1,1,0,0,0,1,0,0,1,0,1,0,0,1,0,1,0,0,-1,0,0]
        elif CrystalType == "tetragonal1" or CrystalType == "t1":
            CoefCoef = [1,1,1,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,1,0,0,0,1,0,0,0,1,
                0,0,1,0,1,1,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,0]        
        elif CrystalType == "rhombohedral2" or CrystalType == "r2":
            CoefCoef = [1,1,1,1,1,1,A,1,1,1,-1,1,1,1,0,1,1,B,1,C,D,1,1,E,F,1,1,-1,-1,0,
                1,-1,G,1,H,I,1,0,0,0,1,0,-1,1,1,J,1,1,1,-1,K,1,-1,-1,1,-1]
        elif CrystalType == "rhombohedral1" or CrystalType == "r1":
            CoefCoef = [1,1,1,1,0,0,A,1,1,0,0,1,1,0,0,1,0,0,1,C,D,1,1,E,0,0,1,-1,0,0,1,
                0,0,1,H,I,1,0,0,0,1,0,0,1,1,J,1,0,0,-1,K,1,0,0,0,0]
        elif CrystalType == "hexagonal2" or CrystalType == "h2":
            CoefCoef = [1,1,1,0,0,1,A,1,0,0,-1,1,0,0,0,1,1,0,1,0,D,1,1,0,0,1,1,0,0,0,1,
                -1,0,1,0,I,1,0,0,0,1,0,0,1,0,J,0,0,1,0,K,0,0,-1,0,-1]
        elif CrystalType == "hexagonal1" or CrystalType == "h1":
            CoefCoef = [1,1,1,0,0,0,A,1,0,0,0,1,0,0,0,1,0,0,1,0,D,1,1,0,0,0,1,0,0,0,1,
                0,0,1,0,I,1,0,0,0,1,0,0,1,0,J,0,0,0,0,K,0,0,0,0,0]        
        elif CrystalType == "cubic2" or CrystalType == "c2":
            CoefCoef = [1,1,1,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,1,0,0,0,1,0,0,0,1,
                0,0,1,0,1,1,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,0]
        elif CrystalType == "cubic1" or CrystalType == "c1":
            CoefCoef = [1,1,1,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,1,0,0,0,1,0,0,0,1,
                0,0,1,0,1,1,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,0]
        elif CrystalType == "isotropic" or CrystalType == "i":
            CoefCoef = [1,1,1,0,0,0,1,1,0,0,0,1,0,0,0,L,0,0,M,0,M,1,1,0,0,0,1,0,0,0,M,
                0,0,L,0,M,1,0,0,0,M,0,0,M,0,L,0,0,0,0,N,0,0,0,0,0]
    elif Ord == 4:
        AA=[-0.125,0.375,-2,0.75];AB=[-1./3.,-4./3.];AC=[-1./3.,4./3.];AD=[-1./3.];AE=[1./6.,0.5];AF=[-1./6.,-0.5]
        AG=[-0.1875,0.3125,1./3.,-0.125];AH=[0.375,-0.125,-2,0.75];AI=[1,-1,1];AJ=[-1./3.,4./3.];AK=[-1./3.,-4./3.]
        AL=[-1./3.];AM=[1,-2];AN=[-0.5,-0.5];AO=[-0.5,-0.5];AP=[0.5,2,-1.5,-2];AQ=[-1./3.,2./3.]
        AR=[-1./6.,1./6.,-2./3.];AS=[0.5,1,-0.5,-2];AT=[1./6.,-1./6.,-2./3.];AU=[0.0625,0.0625,-1./3.,-0.125]
        AV=[0.25,0.75];AW=[-0.25,-0.75];AX=[-0.5,0.75,-0.25];AY=[-1./3.,-2./3.];AZ=[-1./3.,-2./3.]
        BA=[-0.25,-0.5,0.75];BB=[-1./6.,-1./6.,1./3.];BC=[-1./6.,-1./6.,-1./3.];BD=[-1./3.];BE=[-1./3.,-4./3.]
        BF=[-0.5,-1./6.];BG=[1,1,-1];BH=[0.5,1./6.];BI=[0.3125,-0.1875,1./3.,-0.125];BJ=[-0.75,-0.25]
        BK=[0.75,0.25];BL=[0.5,-0.25,-0.25];BM=[-2./3.,-1./3.];BN=[2./3.,-1./3.];BO=[-2./3.,-1./3.]
        BP=[-0.25,0.5,-0.25];BQ=[-1./6.,-1./6.,-1./3.];BR=[-2./3.,1./3.];BS=[-1./6.,-1./6.,1./3.];BT=[-1./3.]
        BU=[-0.5,0.5];BV=[-0.5,-0.5];BW=[-0.5,-0.5];BX=[-0.5,0.5];BY=[1./3.];BZ=[1./6.,-1./6.];CA=[-1./6.,1./6.]
        CB=[-1./3.,2./3.];CC=[0.5,-0.5];CD=[-1,1,1];CE=[0.25,-2,0.75];CF=[1,-2];CG=[0.125,1./3.,-0.125]
        CH=[0.0625,-0.5,-0.0625,0.5];CI=[0.125,-1./3.,-0.125];CJ=[0.03125,1./12.,-0.03125,-0.25]
        if CrystalType == "triclinic" or CrystalType == "n":
           CoefCoef = [1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
                1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
                1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1]
        elif CrystalType == "monoclinic" or CrystalType == "m":
            CoefCoef = [1,1,1,0,0,1,1,1,0,0,1,1,0,0,1,1,1,0,1,0,1,1,1,0,0,1,1,0,0,1,1,1,0,1,0,1,1,0,0,1,1,
                1,0,1,0,1,0,0,1,0,1,0,0,1,0,1,1,1,0,0,1,1,0,0,1,1,1,0,1,0,1,1,0,0,1,1,1,0,1,0,1,0,0,1,0,1,
                0,0,1,0,1,1,0,0,1,1,1,0,1,0,1,0,0,1,0,1,0,0,1,0,1,1,1,0,1,0,1,1,0,1,0,1,0,1,0,1]
        elif CrystalType == "orthorhombic" or CrystalType == "o":
            CoefCoef = [1,1,1,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,0,0,0,1,
                0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,0,0,0,1,0,0,1,0,1,0,0,0,0,1,
                0,0,0,0,0,1,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,1,0,1]
        elif CrystalType == "tetragonal2" or CrystalType == "t2":
            CoefCoef = [1,1,1,0,0,1,1,1,0,0,1,1,0,0,1,1,1,0,1,0,1,1,1,0,0,-1,1,0,0,0,1,0,0,1,0,1,1,0,0,1,1,
                1,0,1,0,1,0,0,1,0,1,0,0,1,0,1,1,1,0,0,-1,1,0,0,-1,1,-1,0,1,0,1,1,0,0,-1,1,-1,0,1,0,1,0,0,
                -1,0,1,0,0,-1,0,-1,1,0,0,0,1,0,0,1,0,1,0,0,1,0,1,0,0,-1,0,0,1,1,0,1,0,1,-1,0,0,0,1,0,1,0,1]
        elif CrystalType == "tetragonal1" or CrystalType == "t1":
            CoefCoef = [1,1,1,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,0,0,0,1,
                0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,0,0,0,1,0,0,1,0,1,0,0,0,0,1,
                0,0,0,0,0,1,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,1,0,1]
        elif CrystalType == "rhombohedral2" or CrystalType == "r2":
            CoefCoef = [1,AA,1,1,1,1,1,1,AB,AC,AD,1,1,1,-1,1,1,AE,1,AF,AG,AH,AI,AJ,AK,AL,AM,AN,AO,1,AP,AQ,
                AR,AS,AT,AU,1,1,1,0,1,1,AV,1,AW,AX,1,AY,1,AZ,BA,BB,1,-1,BC,BD,1,1,1,1,1,1,1,1,-1,1,BE,BF,
                BG,BH,BI,1,-1,-1,0,1,-1,BJ,1,BK,BL,1,BM,BN,BO,BP,BQ,1,BR,BS,BT,1,0,0,0,1,0,-1,1,1,1,1,-1,1,
                -1,BU,BV,1,-1,BW,1,1,0,BX,BY,BZ,1,0,CA,CB,1,1,CC,CD,1,1]
        elif CrystalType == "rhombohedral1" or CrystalType == "r1":
            CoefCoef = [1,AA,1,1,0,0,1,1,AB,0,0,1,1,0,0,1,0,0,1,AF,AG,AH,AI,AJ,0,0,AM,AN,0,0,AP,0,0,AS,AT,
                AU,1,1,0,0,1,0,0,1,AW,AX,1,0,0,AZ,BA,BB,0,0,0,0,1,1,1,0,0,1,1,0,0,1,0,0,BG,BH,BI,1,-1,0,0,
                1,0,0,1,BK,BL,1,0,0,BO,BP,BQ,0,0,0,0,1,0,0,0,1,0,0,1,1,1,1,0,0,-1,BU,BV,0,0,0,0,1,0,0,BY,
                BZ,1,0,0,0,0,1,CC,CD,1,1]
        elif CrystalType == "hexagonal2" or CrystalType == "h2":
            CoefCoef = [1,AA,1,0,0,1,1,1,0,0,AD,1,0,0,-1,1,1,0,1,0,AG,AH,AI,0,0,AL,AM,0,0,1,AP,AQ,0,AS,0,
                AU,1,0,0,0,1,1,0,1,0,AX,0,0,1,0,BA,0,0,-1,0,BD,1,1,0,0,1,1,0,0,-1,1,BE,0,BG,0,BI,1,0,0,
                0,1,-1,0,1,0,BL,0,0,BN,0,BP,0,0,BR,0,BT,1,0,0,0,1,0,0,1,0,1,0,0,1,0,BU,0,0,-1,0,1,1,0,0,
                BY,0,1,0,0,CB,0,1,0,CD,0,1]
        elif CrystalType == "hexagonal1" or CrystalType == "h1":
            CoefCoef = [1,AA,1,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,AG,AH,AI,0,0,0,AM,0,0,0,AP,0,0,AS,0,AU,1,0,
                0,0,1,0,0,1,0,AX,0,0,0,0,BA,0,0,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,BG,0,BI,1,0,0,0,1,0,0,1,0,BL,
                0,0,0,0,BP,0,0,0,0,0,1,0,0,0,1,0,0,1,0,1,0,0,0,0,BU,0,0,0,0,0,1,0,0,BY,0,1,0,0,0,0,1,0,CD,0,1]
        elif CrystalType == "cubic2" or CrystalType == "c2":
            CoefCoef = [1,1,1,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,0,0,0,1,
                0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,0,0,0,1,0,0,1,0,1,0,0,0,0,1,
                0,0,0,0,0,1,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,1,0,1]
        elif CrystalType == "cubic1" or CrystalType == "c1":
            CoefCoef = [1,1,1,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,0,0,0,1,
                0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,0,0,0,1,0,0,1,0,1,0,0,0,0,1,
                0,0,0,0,0,1,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,1,0,1]
        elif CrystalType == "isotropic" or CrystalType == "i":
            CoefCoef = [1,CE,1,0,0,0,1,CF,0,0,0,1,0,0,0,1,0,0,CG,0,1,1,1,0,0,0,1,0,0,0,CH,0,0,1,0,CI,1,0,
                0,0,1,0,0,1,0,1,0,0,0,0,CJ,0,0,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,0,0,0,1,0,0,1,0,1,0,
                0,0,0,1,0,0,0,0,0,1,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,0,0,BY,0,1,0,0,0,0,1,0,1,0,1]
    elif Ord == 5:
        if CrystalType == "triclinic" or CrystalType == "n":
            CoefCoef = [1] * 252
        elif CrystalType == "monoclinic" or CrystalType == "m":
            CoefCoef = [1,1,1,0,0,1,1,1,0,0,1,1,0,0,1,1,1,0,1,0,1,1,1,0,0,1,1,0,0,1,1,1,0,1,0,1,1,0,0,1,1,
                        1,0,1,0,1,0,0,1,0,1,0,0,1,0,1,1,1,0,0,1,1,0,0,1,1,1,0,1,0,1,1,0,0,1,1,1,0,1,0,1,0,
                        0,1,0,1,0,0,1,0,1,1,0,0,1,1,1,0,1,0,1,0,0,1,0,1,0,0,1,0,1,1,1,0,1,0,1,1,0,1,0,1,0,
                        1,0,1,1,1,0,0,1,1,0,0,1,1,1,0,1,0,1,1,0,0,1,1,1,0,1,0,1,0,0,1,0,1,0,0,1,0,1,1,0,0,
                        1,1,1,0,1,0,1,0,0,1,0,1,0,0,1,0,1,1,1,0,1,0,1,1,0,1,0,1,0,1,0,1,1,0,0,1,1,1,0,1,0,
                        1,0,0,1,0,1,0,0,1,0,1,1,1,0,1,0,1,1,0,1,0,1,0,1,0,1,0,0,1,0,1,0,0,1,0,1,0,1,0,1,0,
                        0,1,0,1,0,1]
        elif CrystalType == "orthorhombic" or CrystalType == "o":
            CoefCoef = [1,1,1,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,0,0,0,1,
                        0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,0,0,0,1,0,0,1,0,1,0,
                        0,0,0,1,0,0,0,0,0,1,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,
                        1,0,1,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,0,0,
                        0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,1,0,1,1,0,0,0,1,0,0,1,0,
                        1,0,0,0,0,1,0,0,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,1,0,1,0,0,0,0,1,0,0,0,0,0,0,1,0,1,0,
                        0,0,0,0,0,0]
        elif CrystalType == "tetragonal2" or CrystalType == "t2":
            CoefCoef = [1,1,1,0,0,1,1,1,0,0,1,1,0,0,1,1,1,0,1,0,1,1,1,0,0,0,1,0,0,1,1,1,0,1,0,1,1,0,0,1,1,
                        1,0,1,0,1,0,0,1,0,1,0,0,1,0,1,1,1,0,0,-1,1,0,0,-1,1,-1,0,1,0,1,1,0,0,0,1,0,0,1,0,
                        1,0,0,1,0,1,0,0,-1,0,0,1,0,0,1,1,1,0,1,0,1,0,0,1,0,1,0,0,1,0,1,1,1,0,1,0,1,1,0,1,
                        0,1,0,1,0,1,1,1,0,0,-1,1,0,0,-1,1,-1,0,1,0,1,1,0,0,-1,1,-1,0,1,0,1,0,0,-1,0,1,0,0,
                        -1,0,-1,1,0,0,-1,1,-1,0,1,0,1,0,0,-1,0,1,0,0,-1,0,-1,1,-1,0,1,0,1,-1,0,-1,0,1,0,1,
                        0,1,1,0,0,0,1,0,0,1,0,1,0,0,1,0,1,0,0,1,0,0,1,1,0,1,0,1,-1,0,0,0,1,0,1,0,1,0,0,1,
                        0,1,0,0,0,0,1,0,1,0,1,0,0,-1,0,-1,0,0]
        elif CrystalType == "tetragonal1" or CrystalType == "t1":
            CoefCoef = [1,1,1,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,0,0,0,1,
                        0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,0,0,0,1,0,0,1,0,1,0,
                        0,0,0,1,0,0,0,0,0,1,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,
                        1,0,1,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,0,0,
                        0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,1,0,1,1,0,0,0,1,0,0,1,0,
                        1,0,0,0,0,1,0,0,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,1,0,1,0,0,0,0,1,0,0,0,0,0,0,1,0,1,0,
                        0,0,0,0,0,0]
        elif CrystalType == "rhombohedral2" or CrystalType == "r2":
            CoefCoef = []
            raise ValueError('Currently not supported.') 
        elif CrystalType == "rhombohedral1" or CrystalType == "r1":
            CoefCoef = []
            raise ValueError('Currently not supported.') 
        elif CrystalType == "hexagonal2" or CrystalType == "h2":
            CoefCoef = []
            raise ValueError('Currently not supported.') 
        elif CrystalType == "hexagonal1" or CrystalType == "h1":
            CoefCoef = []
            raise ValueError('Currently not supported.')         
        elif CrystalType == "cubic2" or CrystalType == "c2":
            CoefCoef = [1,1,1,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,0,0,0,1,
                        0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,0,0,0,1,0,0,1,0,1,0,
                        0,0,0,1,0,0,0,0,0,1,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,
                        1,0,1,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,0,0,
                        0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,1,0,1,1,0,0,0,1,0,0,1,0,
                        1,0,0,0,0,1,0,0,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,1,0,1,0,0,0,0,1,0,0,0,0,0,0,1,0,1,0,
                        0,0,0,0,0,0]
        elif CrystalType == "cubic1" or CrystalType == "c1":
            CoefCoef = [1,1,1,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,0,0,0,1,
                        0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,0,0,0,1,0,0,1,0,1,0,
                        0,0,0,1,0,0,0,0,0,1,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,
                        1,0,1,1,1,0,0,0,1,0,0,0,1,0,0,1,0,1,1,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,0,0,
                        0,1,0,0,1,0,1,0,0,0,0,1,0,0,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,1,0,1,1,0,0,0,1,0,0,1,0,
                        1,0,0,0,0,1,0,0,0,0,0,1,0,0,1,0,1,0,0,0,0,1,0,1,0,1,0,0,0,0,1,0,0,0,0,0,0,1,0,1,0,
                        0,0,0,0,0,0]
    return CoefCoef

def coef_ind(CrystalType = "c1", Ord = 3):
    #ATTENTION: here the first element is 1, not 0
    '''
    Get the index of independent elastic constants.
        Note: this function is always used with coef_crystal function 
              and the example is shown in coef_crystal function)
    Parameter
    ---------
        CrystalType: str
            The crystal type, casse insensitive (Both capital or lower case are ok)
        Ord: int
            The order of elastic constants
    Return
    ------
        CoefInd: list
            The index under specific crystal type and order
    '''
    #for 2nd elastic constant
    CrystalType = CrystalType.lower()
    if Ord == 2:
        A = [1, 2]
        if CrystalType == "triclinic" or CrystalType == "n":
            CoefInd = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21]
        elif CrystalType == "monoclinic" or CrystalType == "m":
            CoefInd = [1,2,3,0,5,0,7,8,0,10,0,12,0,14,0,16,0,18,19,0,21]
        elif CrystalType == "orthorhombic" or CrystalType == "o":
            CoefInd = [1,2,3,0,0,0,7,8,0,0,0,12,0,0,0,16,0,0,19,0,21]
        elif CrystalType == "tetragonal2" or CrystalType == "t2":
            CoefInd = [1,2,3,0,0,6,1,3,0,0,6,12,0,0,0,16,0,0,16,0,21]
        elif CrystalType == "tetragonal1" or CrystalType == "t1":
            CoefInd = [1,2,3,0,0,0,1,3,0,0,0,12,0,0,0,16,0,0,16,0,21]        
        elif CrystalType == "rhombohedral2" or CrystalType == "r2":
            CoefInd = [1,2,3,4,5,0,1,3,4,5,0,12,0,0,0,16,0,5,16,4,A]
        elif CrystalType == "rhombohedral1" or CrystalType == "r1":
            CoefInd = [1,2,3,4,0,0,1,3,4,0,0,12,0,0,0,16,0,0,16,4,A]
        elif CrystalType == "hexagonal2" or CrystalType == "h2":
            CoefInd = [1,2,3,0,0,0,1,3,0,0,0,12,0,0,0,16,0,0,16,0,A]
        elif CrystalType == "hexagonal1" or CrystalType == "h1":
            CoefInd = [1,2,3,0,0,0,1,3,0,0,0,12,0,0,0,16,0,0,16,0,A]
        elif CrystalType == "cubic2" or CrystalType == "c2":
            CoefInd = [1,2,2,0,0,0,1,2,0,0,0,1,0,0,0,16,0,0,16,0,16]
        elif CrystalType == "cubic1" or CrystalType == "c1":
            CoefInd = [1,2,2,0,0,0,1,2,0,0,0,1,0,0,0,16,0,0,16,0,16]
        elif CrystalType == "isotropic" or CrystalType == "i":
            CoefInd = [1,2,2,0,0,0,1,2,0,0,0,1,0,0,0,A,0,0,A,0,A]
    elif Ord == 3:
        A = [1, 2, 22];B = [5, 10];C = [4, 9];D = [1, 2, 22];E = [4, 9];F = [5, 10];G = [5, 10]
        H = [4, 9];I = [1, 2, 22];J = [3, 8];K = [16, 19];L = [2, 8,];M = [1, 2];N = [1, 2, 8]
        if CrystalType == "triclinic" or CrystalType == "n":
            CoefInd = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,
                29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56]
        elif CrystalType == "monoclinic" or CrystalType == "m":
            CoefInd = [1,2,3,0,5,0,7,8,0,10,0,12,0,14,0,16,0,18,19,0,21,22,23,0,25,0,27,
                0,29,0,31,0,33,34,0,36,37,0,39,0,41,0,43,44,0,46,0,48,0,0,51,0,53,0,55,0]
        elif CrystalType == "orthorhombic" or CrystalType == "o":
            CoefInd = [1,2,3,0,0,0,7,8,0,0,0,12,0,0,0,16,0,0,19,0,21,22,23,0,0,0,27,0,0,
                0,31,0,0,34,0,36,37,0,0,0,41,0,0,44,0,46,0,0,0,0,51,0,0,0,0,0]
        elif CrystalType == "tetragonal2" or CrystalType == "t2":
            CoefInd = [1,2,3,0,0,6,2,8,0,0,0,12,0,0,15,16,17,0,19,0,21,1,3,0,0,6,12,0,0,
                15,19,17,0,16,0,21,37,0,0,0,41,0,0,41,0,46,0,0,49,0,51,0,0,49,0,0]
        elif CrystalType == "tetragonal1" or CrystalType == "t1":
            CoefInd = [1,2,3,0,0,0,2,8,0,0,0,12,0,0,0,16,0,0,19,0,21,1,3,0,0,0,12,0,0,0,
                19,0,0,16,0,21,37,0,0,0,41,0,0,41,0,46,0,0,0,0,51,0,0,0,0,0]        
        elif CrystalType == "rhombohedral2" or CrystalType == "r2":
            CoefInd = [1,2,3,4,5,6,A,8,9,10,6,12,13,14,0,16,17,B,19,C,D,22,3,E,F,6,12,13,
                14,0,19,17,G,16,H,I,37,0,0,0,41,0,14,41,13,J,47,48,17,47,K,9,48,17,10,6]
        elif CrystalType == "rhombohedral1" or CrystalType == "r1":
            CoefInd = [1,2,3,4,0,0,A,8,9,0,0,12,13,0,0,16,0,0,19,C,D,22,3,E,0,0,12,13,0,0,
                19,0,0,16,H,I,37,0,0,0,41,0,0,41,13,J,47,0,0,47,K,9,0,0,0,0]
        elif CrystalType == "hexagonal2" or CrystalType == "h2":
            CoefInd = [1,2,3,0,0,6,A,8,0,0,6,12,0,0,0,16,17,0,19,0,D,22,3,0,0,6,12,0,0,0,
                19,17,0,16,0,I,37,0,0,0,41,0,0,41,0,J,0,0,17,0,K,0,0,17,0,6]
        elif CrystalType == "hexagonal1" or CrystalType == "h1":
            CoefInd = [1,2,3,0,0,0,A,8,0,0,0,12,0,0,0,16,0,0,19,0,D,22,3,0,0,0,12,0,0,0,
                19,0,0,16,0,I,37,0,0,0,41,0,0,41,0,J,0,0,0,0,K,0,0,0,0,0]
        elif CrystalType == "cubic2" or CrystalType == "c2":
            CoefInd = [1,2,3,0,0,0,3,8,0,0,0,2,0,0,0,16,0,0,19,0,21,1,2,0,0,0,3,0,0,0,21,
                0,0,16,0,19,1,0,0,0,19,0,0,21,0,16,0,0,0,0,51,0,0,0,0,0]
        elif CrystalType == "cubic1" or CrystalType == "c1":
            CoefInd = [1,2,2,0,0,0,2,8,0,0,0,2,0,0,0,16,0,0,19,0,19,1,2,0,0,0,2,0,0,0,19,
                0,0,16,0,19,1,0,0,0,19,0,0,19,0,16,0,0,0,0,51,0,0,0,0,0]
        elif CrystalType == "isotropic" or CrystalType == "i":
            CoefInd = [1,2,2,0,0,0,2,8,0,0,0,2,0,0,0,L,0,0,M,0,M,1,2,0,0,0,2,0,0,0,M,0,0,
                L,0,M,1,0,0,0,M,0,0,M,0,L,0,0,0,0,N,0,0,0,0,0]
    elif Ord == 4:
        AA=[1,57,126,7];AB=[59,125];AC=[60,121];AD=[6];AE=[5,60];AF=[4,59];AG=[1,57,126,7];AH=[1,57,126,7]
        AI=[3,58,8];AJ=[4,125];AK=[5,121];AL=[6];AM=[12,101];AN=[13,63];AO=[14,64];AP=[16,19,66,117]
        AQ=[17,49];AR=[5,60,121];AS=[16,19,66,117];AT=[4,59,125];AU=[1,57,126,7];AV=[14,64];AW=[13,63]
        AX=[3,58,8];AY=[53,88];AZ=[47,82];BA=[16,19,66];BB=[4,59,125];BC=[5,60,121];BD=[6];BE=[17,49]
        BF=[5,60];BG=[16,19,66];BH=[4,59];BI=[1,57,126,7];BJ=[14,64];BK=[13,63];BL=[3,58,8];BM=[53,88]
        BN=[17,49];BO=[47,82];BP=[16,19,66];BQ=[4,59,125];BR=[17,49];BS=[5,60,121];BT=[6];BU=[41,44]
        BV=[13,63];BW=[14,64];BX=[53,88];BY=[112];BZ=[47,82];CA=[53,88];CB=[17,49];CC=[47,82]
        CD=[19,66,117];CE=[1,112,7];CF=[7,16];CG=[1,112,7];CH=[1,112,7,16];CI=[1,112,7];CJ=[1,112,7,16]
        if CrystalType == "triclinic" or CrystalType == "n":
            CoefInd = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,
                30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,
                59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,
                88,89,90,91,92,93,94,95,96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,
                113,114,115,116,117,118,119,120,121,122,123,124,125,126]
        elif CrystalType == "monoclinic" or CrystalType == "m":
            CoefInd = [1,2,3,0,0,6,7,8,0,0,11,12,0,0,15,16,17,0,19,0,21,22,23,0,0,26,27,0,0,30,31,32,
                0,34,0,36,37,0,0,40,41,42,0,44,0,46,0,0,49,0,51,0,0,54,0,56,57,58,0,0,61,62,0,0,65,66,
                67,0,69,0,71,72,0,0,75,76,77,0,79,0,81,0,0,84,0,86,0,0,89,0,91,92,0,0,95,96,97,0,99,0,
                101,0,0,104,0,106,0,0,109,0,111,112,113,0,115,0,117,118,0,120,0,122,0,124,0,126]
        elif CrystalType == "orthorhombic" or CrystalType == "o":
            CoefInd = [1,2,3,0,0,0,7,8,0,0,0,12,0,0,0,16,0,0,19,0,21,22,23,0,0,0,27,0,0,0,31,0,0,34,
                0,36,37,0,0,0,41,0,0,44,0,46,0,0,0,0,51,0,0,0,0,0,57,58,0,0,0,62,0,0,0,66,0,0,69,0,71,
                72,0,0,0,76,0,0,79,0,81,0,0,0,0,86,0,0,0,0,0,92,0,0,0,96,0,0,99,0,101,0,0,0,0,106,0,0,
                0,0,0,112,0,0,115,0,117,0,0,0,0,122,0,124,0,126]
        elif CrystalType == "tetragonal2" or CrystalType == "t2":
            CoefInd = [1,2,3,0,0,6,7,8,0,0,11,12,0,0,15,16,17,0,19,0,21,2,8,0,0,11,27,0,0,0,31,0,0,
                31,0,36,37,0,0,40,41,42,0,44,0,46,0,0,49,0,51,0,0,54,0,56,1,3,0,0,6,12,0,0,15,19,17,0,
                16,0,21,37,0,0,40,44,42,0,41,0,46,0,0,54,0,51,0,0,49,0,56,92,0,0,0,96,0,0,96,0,101,0,0,
                104,0,106,0,0,104,0,0,112,113,0,115,0,117,113,0,0,0,112,0,117,0,126]
        elif CrystalType == "tetragonal1" or CrystalType == "t1":
            CoefInd = [1,2,3,0,0,0,7,8,0,0,0,12,0,0,0,16,0,0,19,0,21,2,8,0,0,0,27,0,0,0,31,0,0,31,0,
                36,37,0,0,0,41,0,0,44,0,46,0,0,0,0,51,0,0,0,0,0,1,3,0,0,0,12,0,0,0,19,0,0,16,0,21,37,0,
                0,0,44,0,0,41,0,46,0,0,0,0,51,0,0,0,0,0,92,0,0,0,96,0,0,96,0,101,0,0,0,0,106,0,0,0,0,0,
                112,0,0,115,0,117,0,0,0,0,112,0,117,0,126]
        elif CrystalType == "rhombohedral2" or CrystalType == "r2":
            CoefInd = [1,AA,3,4,5,6,7,8,AB,AC,AD,12,13,14,111,16,17,AE,19,AF,AG,AH,AI,AJ,AK,AL,AM,AN,
                AO,111,AP,AQ,AR,AS,AT,AU,37,38,39,0,41,104,AV,44,AW,AX,47,AY,49,AZ,BA,BB,53,49,BC,BD,57,
                58,59,60,6,12,63,64,111,66,BE,BF,BG,BH,BI,37,38,39,0,44,104,BJ,41,BK,BL,82,BM,BN,BO,BP,
                BQ,88,BR,BS,BT,92,0,0,0,96,0,39,96,38,101,102,108,104,102,BU,BV,108,104,BW,111,112,0,BX,
                BY,BZ,117,0,CA,CB,121,112,CC,CD,125,126]
        elif CrystalType == "rhombohedral1" or CrystalType == "r1":
            CoefInd = [1,AA,3,4,0,0,7,8,AB,0,0,12,13,0,0,16,0,0,19,AF,AG,AH,AI,AJ,0,0,AM,AN,0,0,AP,0,
                0,AS,AT,AU,37,38,0,0,41,0,0,44,AW,AX,47,0,0,AZ,BA,BB,0,0,0,0,57,58,59,0,0,12,63,0,0,66,0,
                0,BG,BH,BI,37,38,0,0,44,0,0,41,BK,BL,82,0,0,BO,BP,BQ,0,0,0,0,92,0,0,0,96,0,0,96,38,101,
                102,0,0,102,BU,BV,0,0,0,0,112,0,0,BY,BZ,117,0,0,0,0,112,CC,CD,125,126]
        elif CrystalType == "hexagonal2" or CrystalType == "h2":
            CoefInd = [1,AA,3,0,0,6,7,8,0,0,AD,12,0,0,111,16,17,0,19,0,AG,AH,AI,0,0,AL,AM,0,0,111,AP,
                AQ,0,AS,0,AU,37,0,0,0,41,104,0,44,0,AX,0,0,49,0,BA,0,0,49,0,BD,57,58,0,0,6,12,0,0,111,
                66,BE,0,BG,0,BI,37,0,0,0,44,104,0,41,0,BL,0,0,BN,0,BP,0,0,BR,0,BT,92,0,0,0,96,0,0,96,0,
                101,0,0,104,0,BU,0,0,104,0,111,112,0,0,BY,0,117,0,0,CB,0,112,0,CD,0,126]
        elif CrystalType == "hexagonal1" or CrystalType == "h1":
            CoefInd = [1,AA,3,0,0,0,7,8,0,0,0,12,0,0,0,16,0,0,19,0,AG,AH,AI,0,0,0,AM,0,0,0,AP,0,0,AS,0,
                AU,37,0,0,0,41,0,0,44,0,AX,0,0,0,0,BA,0,0,0,0,0,57,58,0,0,0,12,0,0,0,66,0,0,BG,0,BI,37,
                0,0,0,44,0,0,41,0,BL,0,0,0,0,BP,0,0,0,0,0,92,0,0,0,96,0,0,96,0,101,0,0,0,0,BU,0,0,0,0,0,
                112,0,0,BY,0,117,0,0,0,0,112,0,CD,0,126]
        elif CrystalType == "cubic2" or CrystalType == "c2":
            CoefInd = [1,2,3,0,0,0,7,8,0,0,0,7,0,0,0,16,0,0,19,0,21,3,8,0,0,0,8,0,0,0,31,0,0,34,0,36,
                2,0,0,0,34,0,0,36,0,31,0,0,0,0,51,0,0,0,0,0,1,2,0,0,0,7,0,0,0,21,0,0,16,0,19,3,0,0,0,36,
                0,0,31,0,34,0,0,0,0,51,0,0,0,0,0,1,0,0,0,19,0,0,21,0,16,0,0,0,0,51,0,0,0,0,0,112,0,0,115,
                0,115,0,0,0,0,112,0,115,0,112]
        elif CrystalType == "cubic1" or CrystalType == "c1":
            CoefInd = [1,2,2,0,0,0,7,8,0,0,0,7,0,0,0,16,0,0,19,0,19,2,8,0,0,0,8,0,0,0,31,0,0,31,0,36,
                2,0,0,0,31,0,0,36,0,31,0,0,0,0,51,0,0,0,0,0,1,2,0,0,0,7,0,0,0,19,0,0,16,0,19,2,0,0,0,36,
                0,0,31,0,31,0,0,0,0,51,0,0,0,0,0,1,0,0,0,19,0,0,19,0,16,0,0,0,0,51,0,0,0,0,0,112,0,0,115,
                0,115,0,0,0,0,112,0,115,0,112]
        elif CrystalType == "isotropic" or CrystalType == "i":
            CoefInd = [1,CE,2,0,0,0,7,CF,0,0,0,7,0,0,0,16,0,0,CG,0,19,2,8,0,0,0,8,0,0,0,CH,0,0,31,0,
                CI,2,0,0,0,31,0,0,36,0,31,0,0,0,0,CJ,0,0,0,0,0,1,2,0,0,0,7,0,0,0,19,0,0,16,0,19,2,0,0,0,
                36,0,0,31,0,31,0,0,0,0,51,0,0,0,0,0,1,0,0,0,19,0,0,19,0,16,0,0,0,0,51,0,0,0,0,0,112,0,0,
                BY,0,115,0,0,0,0,112,0,115,0,112]
    elif Ord == 5:
        if CrystalType == "triclinic" or CrystalType == "n":
            CoefInd = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,
                       31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,
                       58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,
                       85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,101,102,103,104,105,106,107,108,
                       109,110,111,112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127,128,
                       129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,144,145,146,147,148,
                       149,150,151,152,153,154,155,156,157,158,159,160,161,162,163,164,165,166,167,168,
                       169,170,171,172,173,174,175,176,177,178,179,180,181,182,183,184,185,186,187,188,
                       189,190,191,192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207,208,
                       209,210,211,212,213,214,215,216,217,218,219,220,221,222,223,224,225,226,227,228,
                       229,230,231,232,233,234,235,236,237,238,239,240,241,242,243,244,245,246,247,248,
                       249,250,251,252]
        elif CrystalType == "monoclinic" or CrystalType == "m":
            CoefInd = [1,2,3,0,0,6,7,8,0,0,11,12,0,0,15,16,17,0,19,0,21,22,23,0,0,26,27,0,0,30,31,32,0,
                       34,0,36,37,0,0,40,41,42,0,44,0,46,0,0,49,0,51,0,0,54,0,56,57,58,0,0,61,62,0,0,65,
                       66,67,0,69,0,71,72,0,0,75,76,77,0,79,0,81,0,0,84,0,86,0,0,89,0,91,92,0,0,95,96,97,
                       0,99,0,101,0,0,104,0,106,0,0,109,0,111,112,113,0,115,0,117,118,0,120,0,122,0,124,
                       0,126,127,128,0,0,131,132,0,0,135,136,137,0,139,0,141,142,0,0,145,146,147,0,149,0,
                       151,0,0,154,0,156,0,0,159,0,161,162,0,0,165,166,167,0,169,0,171,0,0,174,0,176,0,0,
                       179,0,181,182,183,0,185,0,187,188,0,190,0,192,0,194,0,196,197,0,0,200,201,202,0,
                       204,0,206,0,0,209,0,211,0,0,214,0,216,217,218,0,220,0,222,223,0,225,0,227,0,229,0,
                       231,0,0,234,0,236,0,0,239,0,241,0,243,0,245,0,0,248,0,250,0,252]
        elif CrystalType == "orthorhombic" or CrystalType == "o":
            CoefInd = [1,2,3,0,0,0,7,8,0,0,0,12,0,0,0,16,0,0,19,0,21,22,23,0,0,0,27,0,0,0,31,0,0,34,0,36,
                       37,0,0,0,41,0,0,44,0,46,0,0,0,0,51,0,0,0,0,0,57,58,0,0,0,62,0,0,0,66,0,0,69,0,71,
                       72,0,0,0,76,0,0,79,0,81,0,0,0,0,86,0,0,0,0,0,92,0,0,0,96,0,0,99,0,101,0,0,0,0,106,
                       0,0,0,0,0,112,0,0,115,0,117,0,0,0,0,122,0,124,0,126,127,128,0,0,0,132,0,0,0,136,0,
                       0,139,0,141,142,0,0,0,146,0,0,149,0,151,0,0,0,0,156,0,0,0,0,0,162,0,0,0,166,0,0,
                       169,0,171,0,0,0,0,176,0,0,0,0,0,182,0,0,185,0,187,0,0,0,0,192,0,194,0,196,197,0,0,
                       0,201,0,0,204,0,206,0,0,0,0,211,0,0,0,0,0,217,0,0,220,0,222,0,0,0,0,227,0,229,0,
                       231,0,0,0,0,236,0,0,0,0,0,0,243,0,245,0,0,0,0,0,0,0]
        elif CrystalType == "tetragonal2" or CrystalType == "t2":
            CoefInd = [1,2,3,0,0,6,7,8,0,0,11,12,0,0,15,16,17,0,19,0,21,7,23,0,0,0,27,0,0,30,31,32,0,34,0,
                       36,37,0,0,40,41,42,0,44,0,46,0,0,49,0,51,0,0,54,0,56,2,8,0,0,-11,27,0,0,-30,34,-32,
                       0,31,0,36,72,0,0,0,76,0,0,76,0,81,0,0,84,0,86,0,0,-84,0,0,92,0,0,95,96,97,0,99,0,
                       101,0,0,104,0,106,0,0,109,0,111,112,113,0,115,0,117,118,0,120,0,122,0,124,0,126,1,3,
                       0,0,-6,12,0,0,-15,19,-17,0,16,0,21,12,0,0,-40,44,-42,0,31,0,46,0,0,-54,0,51,0,0,-49,
                       0,-56,92,0,0,-95,99,-97,0,96,0,101,0,0,-109,0,106,0,0,-104,0,-111,122,-118,0,115,0,
                       124,-113,0,-120,0,112,0,117,0,126,197,0,0,0,201,0,0,201,0,206,0,0,209,0,211,0,0,209,
                       0,0,217,218,0,220,0,222,-218,0,0,0,217,0,222,0,231,0,0,234,0,236,0,0,0,0,241,0,236,
                       0,245,0,0,-234,0,-241,0,0]
        elif CrystalType == "tetragonal1" or CrystalType == "t1":
            CoefInd = [1,2,3,0,0,0,7,8,0,0,0,12,0,0,0,16,0,0,19,0,21,7,23,0,0,0,27,0,0,0,31,0,0,34,0,36,
                       37,0,0,0,41,0,0,44,0,46,0,0,0,0,51,0,0,0,0,0,2,8,0,0,0,27,0,0,0,34,0,0,31,0,36,72,
                       0,0,0,76,0,0,76,0,81,0,0,0,0,86,0,0,0,0,0,92,0,0,0,96,0,0,99,0,101,0,0,0,0,106,0,0,
                       0,0,0,112,0,0,115,0,117,0,0,0,0,122,0,124,0,126,1,3,0,0,0,12,0,0,0,19,0,0,16,0,21,
                       12,0,0,0,44,0,0,41,0,46,0,0,0,0,51,0,0,0,0,0,92,0,0,0,99,0,0,96,0,101,0,0,0,0,106,
                       0,0,0,0,0,122,0,0,115,0,124,0,0,0,0,112,0,117,0,126,197,0,0,0,201,0,0,201,0,206,0,
                       0,0,0,211,0,0,0,0,0,217,0,0,220,0,222,0,0,0,0,217,0,222,0,231,0,0,0,0,236,0,0,0,0,
                       0,0,236,0,245,0,0,0,0,0,0,0]   
        elif CrystalType == "rhombohedral2" or CrystalType == "r2":
            CoefInd = []
            raise ValueError('Currently not supported.')  
        elif CrystalType == "rhombohedral1" or CrystalType == "r1":
            CoefInd = []
            raise ValueError('Currently not supported.')  
        elif CrystalType == "hexagonal2" or CrystalType == "h2":
            CoefInd = []
            raise ValueError('Currently not supported.')  
        elif CrystalType == "hexagonal1" or CrystalType == "h1":
            CoefInd = []
            raise ValueError('Currently not supported.')  
        elif CrystalType == "cubic2" or CrystalType == "c2":
            CoefInd = [1,2,3,0,0,0,7,8,0,0,0,12,0,0,0,16,0,0,19,0,21,7,23,0,0,0,23,0,0,0,31,0,0,34,0,36,7,
                        0,0,0,41,0,0,44,0,46,0,0,0,0,51,0,0,0,0,0,2,8,0,0,0,23,0,0,0,46,0,0,41,0,31,8,0,0,
                        0,76,0,0,76,0,76,0,0,0,0,86,0,0,0,0,0,2,0,0,0,34,0,0,36,0,31,0,0,0,0,86,0,0,0,0,0,
                        112,0,0,115,0,117,0,0,0,0,122,0,124,0,126,1,3,0,0,0,7,0,0,0,21,0,0,16,0,19,12,0,0,
                        0,36,0,0,31,0,34,0,0,0,0,51,0,0,0,0,0,3,0,0,0,44,0,0,46,0,41,0,0,0,0,86,0,0,0,0,0,
                        126,0,0,117,0,124,0,0,0,0,112,0,115,0,122,1,0,0,0,19,0,0,21,0,16,0,0,0,0,51,0,0,0,
                        0,0,122,0,0,124,0,115,0,0,0,0,126,0,117,0,115,0,0,0,0,236,0,0,0,0,0,0,236,0,236,0,
                        0,0,0,0,0,0]
        elif CrystalType == "cubic1" or CrystalType == "c1":
            CoefInd = [1,2,2,0,0,0,7,8,0,0,0,7,0,0,0,16,0,0,19,0,19,7,23,0,0,0,23,0,0,0,31,0,0,34,0,36,7,
                       0,0,0,31,0,0,36,0,34,0,0,0,0,51,0,0,0,0,0,2,8,0,0,0,23,0,0,0,34,0,0,31,0,36,8,0,0,
                       0,76,0,0,76,0,76,0,0,0,0,86,0,0,0,0,0,2,0,0,0,34,0,0,36,0,31,0,0,0,0,86,0,0,0,0,0,
                       112,0,0,115,0,115,0,0,0,0,122,0,124,0,122,1,2,0,0,0,7,0,0,0,19,0,0,16,0,19,7,0,0,0,
                       36,0,0,31,0,34,0,0,0,0,51,0,0,0,0,0,2,0,0,0,36,0,0,34,0,31,0,0,0,0,86,0,0,0,0,0,122,
                       0,0,115,0,124,0,0,0,0,112,0,115,0,122,1,0,0,0,19,0,0,19,0,16,0,0,0,0,51,0,0,0,0,0,
                       122,0,0,124,0,115,0,0,0,0,122,0,115,0,112,0,0,0,0,236,0,0,0,0,0,0,236,0,236,0,0,0,
                       0,0,0,0]
    return CoefInd

def get_unique(CoefInd):
    #ATTENTION: here the first element is 1, not 0
    '''
    Get the unique index for given CoefInd
    Parameter
    ---------
        CoefInd: list
            The coef index. Result from coef_ind function
    Return
    ------
        CoefUniq: list
            The index of unique index
    '''
    n = len(CoefInd)
    CoefUniq = []
    for i in range(0, n):
        coefi = CoefInd[i]
        #print(type(coefi))
        try:
            coefi = int(coefi)
            if coefi != 0 :
                if not coefi in CoefUniq:
                    CoefUniq.append(coefi)
        except Exception as e:
            # This one must be dependent
            pass
    return CoefUniq

def group2crytyp(group):
    '''
    Convert point group to Laue's crystal type. This function is not used currently
    Parameter
    ---------
        group: str
            Point group
    Return
    ------
        CrystalType: str
            Laue's crystal type
    '''
    if group == "1" or group == "-1":
        CrystalType = "triclinic"
    elif group == "2" or group == "m" or group == "2/m":
        CrystalType = "monoclinic"
    elif group == "222" or group == "mm2" or group == "222/mmm":
        CrystalType = "orthorhombic"
    elif group == "4" or group == "-4" or group == "4/m":
        CrystalType = "tetragonal2"
    elif group == "422" or group == "4mm" or group == "-42m" or group == "422/mmm":
        CrystalType = "tetragonal1"
    elif group == "23" or group == "2/m-3":
        CrystalType = "cubic2"
    elif group == "432" or group == "-43m" or group == "4/m-32/m":
        CrystalType = "cubic1"
    elif group == "3" or group == "-3":
        CrystalType = "rhombohedral2"
    elif group == "32" or group == "3m" or group == "-32/m":
        CrystalType = "rhombohedral1"
    elif group == "6" or group == "-6" or group == "6/m":
        CrystalType = "hexagonal2"
    elif group == "622" or group == "6mm" or group == "-6m2" or group == "622/mmm":
        CrystalType = "hexagonal1"
    return CrystalType

'''
CrystalType = "h1"
CoefInd = coef_ind(CrystalType, Ord=3)
CoefUniq = get_unique(CoefInd)
print(CoefUniq)
print(len(CoefUniq))
#UniqeCoef = np.unique(IndepCoef)
print(CoefInd)
#print UniqeCoef
print(len(CoefInd))


'''

'''
CrystalType = "r1"
Ord = 4
coefcoef4 = coef_crystal(CrystalType, Ord)
print coefcoef4
print len(coefcoef4)
coefind4 = coef_ind(CrystalType, Ord)
print coefind4
print len(coefind4)
'''
